/*
   Copyright (C) 2003, 2004, Intel Corporation

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qfont.h>
#include <qmath.h>
#include <qpeapplication.h>
#include <qlayout.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpeapplication.h>
#include <qmessagebox.h>
#include <math.h>
#include <unistd.h>

#include "atphone.h"
#include "dialerimpl.h"
#include "resource.h"

DialerImpl::DialerImpl (QWidget * parent, const char *name, WFlags f):
Dialer (parent, name, f)
{
  this->phone = new atphone( this );
  connected = 0;

  numkeys.insert (PushButton1);
  numkeys.insert (PushButton2);
  numkeys.insert (PushButton3);
  numkeys.insert (PushButton4);
  numkeys.insert (PushButton5);
  numkeys.insert (PushButton6);
  numkeys.insert (PushButton7);
  numkeys.insert (PushButton8);
  numkeys.insert (PushButton9);
  numkeys.insert (PushButton10);
  numkeys.insert (PushButton11);
  numkeys.insert (PushButton12);
  connect (&numkeys, SIGNAL (clicked (int)), this, SLOT (numkeysdown (int)));

  ctrlkeys.insert (PushButtonDial);
  ctrlkeys.insert (PushButtonHang);
  ctrlkeys.insert (PushButtonDEL);
  connect (&ctrlkeys, SIGNAL (clicked (int)), this,
	   SLOT (ctrlkeysdown (int)));

  connect( screen, SIGNAL( returnPressed() ), this, SLOT( returnpress() ) );
}

void
DialerImpl::returnpress( void )
{
      if ( connected ) return ;  // We cannot DIAL while already in connected mode
      Dial (screen->text ());

}

void
DialerImpl::numkeysdown (int i)
{
  char num_str[10];

  if (screen->text ().length () >= MAXSIZEOFPHONE && !connected)
    return;

  switch (i)
    {
    case 9:			// "*" key down
      screen->setText (screen->text () + QString ("*"));
      if ( phone->state ==INACALL_STATE ) {
	      phone->generate_DTMFtone( "*" );
      }
      break;
    case 10:			// "0" key down
      screen->setText (screen->text () + QString ("0"));
      if ( phone->state ==INACALL_STATE ) {
	      phone->generate_DTMFtone( "0" );
      }
      break;
    case 11:			// "#" key down
      screen->setText (screen->text () + QString ("#"));
      if ( phone->state ==INACALL_STATE ) {
	      phone->generate_DTMFtone( "#" );
      }
      break;

    default:			// i >= 0 && i<9 , means normal num key down. i should ++
      screen->setText (screen->text () + QString::number (1+i));
      if ( phone->state ==INACALL_STATE ) {
	      memset( num_str, 0, 10 );
	      snprintf( num_str, 10, "%d", i+1 );
	      phone->generate_DTMFtone( num_str );
      }
      break;

    };
  return;
}

void
DialerImpl::ctrlkeysdown (int i)
{
  int leng;
  switch (i)
    {
    case 2:			// "DEL" key down
      if (connected ) break;		// if in connected mode, this DEL key has no use.
      leng = screen->text ().length ();
      if (leng > 0)
	screen->setText (screen->text ().left (leng - 1));
      break;
    case 0:			// "DIAL..." key down
      if ( connected ) break ;  // We cannot DIAL while already in connected mode
      Dial (screen->text ());

      break;
    case 1:			// "HANG UP" key down
      Hangup ();
      break;

    default:
      break;

    };
  return;
}

void
DialerImpl::Dial (QString qstr)		//the qstr is the string to dial
{
  if ( this->phone->state == NEWCALL_STATE ) {
	  this->setCaption( tr("A new call"));
	  this->phone->answer();
  } else {
	  //to write dialing code here
	  if ( qstr.length()<=0 ) {
		  return;
	  }

	  this->setCaption( tr("Dialing ..." ) );
	  this->phone->init();
	  
	  while ( phone->state!=INITED_STATE )
		  sleep(1);

	  this->phone->dial(qstr.ascii());
	  phone->state = DIALING_STATE;
  }
}

void
DialerImpl::Hangup ()
{
  this->setCaption( tr("Hanging up..."));

  //to write hanging code here
  this->phone->hangup();
}

/* call back function */
void 
DialerImpl::Hangedup( )
{
  this->setCaption( tr("Mini dialer" ) );
  this->screen->setText("");
  this->phone->state = INIT_STATE;
  this->connected = 0;
}

void 
DialerImpl::Answer ()
{
	this->setCaption( tr("A new call"));
	if ( this->phone->answer()<0 ) {
		QMessageBox::information (this, "Answer a call",
				  QString (" Failed to answer a call."), "OK");
	}

}

/* callback function */
void
DialerImpl::InACall()
{
	this->phone->state = INACALL_STATE;
	connected = 1;
	this->setCaption( tr("In a call ..." ) );
}

void 
DialerImpl::warning ( char *str)
{
	QMessageBox::information (NULL, "Warning",
				  QString (str), "OK");
}
